//**********************************************************************
// CRhsADOConnection
// =================
//
// Class to encapsulate an ADO Connection
//
// John Rennie
// jrennie@cix.compulink.co.uk
// 15th November 1998
//**********************************************************************

#ifndef STRICT
#define STRICT
#endif

#ifndef INITGUID
#define INITGUID 1
#endif

#include <windows.h>
#include "RhsADO.h"


//**********************************************************************
// CRhsADOConnection::CRhsADOConnection
// CRhsADOConnection::~CRhsADOConnection
// =====================================
// Constructor and destructor
//**********************************************************************

CRhsADOConnection::CRhsADOConnection()
{
  m_Connection = NULL;
}


CRhsADOConnection::~CRhsADOConnection()
{
  if (m_Connection)
    m_Connection->Release();
}


//**********************************************************************
// CRhsADOConnection::Open
// -----------------------
// Open a data source
//**********************************************************************

BOOL CRhsADOConnection::Open(const char* ConnectionString, const char* Username, const char* Password, int Options)
{ HRESULT hr;
  BSTR connection, username, password;

// Check the connection exists

  if (!m_Connection)
    Instantiate();

// Open the ODBC source

  connection = SysAllocStringFromANSI(ConnectionString);
  username =   SysAllocStringFromANSI(Username);
  password =   SysAllocStringFromANSI(Password);

  hr = m_Connection->Open(connection, username, password, Options);

  if (connection)
    SysFreeString(connection);

  if (username)
    SysFreeString(username);

  if (password)
    SysFreeString(password);

  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);

// Return indicating success

  return TRUE;
}


//**********************************************************************
// CRhsADOConnection::Close
// ------------------------
// Close the connection.
// This also releases the ADOConnection object.
//**********************************************************************

void CRhsADOConnection::Close(void)
{
  m_Connection->Close();
  m_Connection->Release();
  m_Connection = NULL;
}


//**********************************************************************
// CRhsADOConnection::Execute
// --------------------------
// Execute a query
//**********************************************************************

CRhsADORecordset& CRhsADOConnection::Execute(const char* Query, int Options, int* Affected)
{ HRESULT hr;
  VARIANT affected;
  ADORecordset* rs = NULL;
  CRhsADORecordset* rhsrs = NULL;

// Do the query

  BSTR query = SysAllocStringFromANSI(Query);
  VariantInit(&affected);
  hr = m_Connection->Execute(query, &affected, adCmdText, &rs);

  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);

  if (Affected)
    *Affected = affected.lVal;

// Return the new recordset object

  rhsrs = new CRhsADORecordset(rs);

  return *rhsrs;
}


//**********************************************************************
// CRhsADOConnection::SetTimeout
// CRhsADOConnection::GetTimeout
// -----------------------------
// Get/set the timeout for commands.
// The timeout is in seconds and the default is 30 seconds.
//**********************************************************************

void CRhsADOConnection::ConnectionTimeout(LONG Timeout)
{ HRESULT hr;

// Check the connection exists

  if (!m_Connection)
    Instantiate();

// Set the timeout

  hr = m_Connection->put_ConnectionTimeout(Timeout);

  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);
}


LONG CRhsADOConnection::ConnectionTimeout(void)
{ LONG l;
  HRESULT hr;

// Check the connection exists

  if (!m_Connection)
    Instantiate();

// Get the timeout

  hr = m_Connection->get_ConnectionTimeout(&l);

  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);

  return l;
}


//**********************************************************************
// CRhsADOConnection::SetTimeout
// CRhsADOConnection::GetTimeout
// -----------------------------
// Get/set the timeout for commands.
// The timeout is in seconds and the default is 30 seconds.
//**********************************************************************

void CRhsADOConnection::CommandTimeout(LONG Timeout)
{ HRESULT hr;

// Check the connection exists

  if (!m_Connection)
    Instantiate();

// Set the timeout

  hr = m_Connection->put_CommandTimeout(Timeout);

  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);
}


LONG CRhsADOConnection::CommandTimeout(void)
{ LONG l;
  HRESULT hr;

// Check the connection exists

  if (!m_Connection)
    Instantiate();

// Get the timeout

  hr = m_Connection->get_CommandTimeout(&l);

  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);

  return l;
}


//**********************************************************************
// CRhsADOConnection::GetLastError
// -------------------------------
// Get the last ADO error
//**********************************************************************

const char* CRhsADOConnection::GetLastError(char* Buffer, int MaxLen)
{ HRESULT hr;
  long numerrs;
  ADOErrors* errors;
  ADOError* error;
  BSTR description;
  CRhsVariant errnum;

// Get the errors collection

  hr = m_Connection->get_Errors(&errors);
  if (FAILED(hr))
  { lstrcpyn(Buffer, "Cannot open Errors collection", MaxLen);
    return Buffer;
  }

// Get the number of errors; if there were no errors return a success message

  errors->get_Count(&numerrs);

  if (numerrs == 0)
  { lstrcpyn(Buffer, "The operation completed successfully", MaxLen);
    errors->Release();
    return Buffer;
  }

// Get the first error (ignore the rest)

  errnum = 0;

  hr = errors->get_Item((VARIANT) errnum, &error);
  if (FAILED(hr))
  { lstrcpyn(Buffer, "Cannot retrieve error number 0", MaxLen);
    errors->Release();
    return Buffer;
  }

// Get the error description

  hr = error->get_Description(&description);
  if (FAILED(hr))
  { lstrcpyn(Buffer, "Cannot retrieve description for error 0", MaxLen);
    error->Release();
    errors->Release();
    return Buffer;
  }
  
  if (!description)
    lstrcpyn(Buffer, "No description for error 0", MaxLen);
  else
    WideCharToMultiByte(CP_ACP, 0, description, -1, Buffer, MaxLen, 0, NULL);

// All done, release the interfaces and return the error

  error->Release();
  errors->Release();

  return Buffer;
}


//**********************************************************************
// CRhsADOConnection::Instantiate
// ------------------------------
// Protected method to instantiate the ADOConnection interface
//**********************************************************************

BOOL CRhsADOConnection::Instantiate(void)
{ HRESULT hr;

// If the ADOConnection already exists there is no need to create it again

  if (m_Connection)
    return TRUE;

// Create the ADOConnection

  hr = CoCreateInstance(CLSID_CADOConnection, NULL, CLSCTX_INPROC_SERVER, IID_IADOConnection, (LPVOID*) &m_Connection);

  if (FAILED(hr))
    throw CRhsADOException(hr, NULL);

// Return indicating success

  return TRUE;
}


//**********************************************************************
// SysAllocStringFromANSI
// ----------------------
//**********************************************************************

BSTR SysAllocStringFromANSI(const char* s)
{ BSTR b;
  WCHAR w[1024];

  if (!s)
    return NULL;

  MultiByteToWideChar(CP_ACP, 0, s, -1, w, 1023);
  b = SysAllocString(w);

  return b;
}


