//**********************************************************************
// adotest
// =======
//
// Execute a query using ADO
//
// John Rennie
// jrennie@cix.compulink.co.uk
// 15th November 1998
//**********************************************************************

#ifndef STRICT
#define STRICT
#endif

#include <windows.h>
#include <stdio.h>
#include <Misc/RhsDate.h>
#include <RhsADO/RhsADO.h>


//**********************************************************************
// Function prototypes
// -------------------
//**********************************************************************

const char* GetLastErrorMessage(void);
BOOL strtoint(const char* s, int* i);


//**********************************************************************
// Global variables
// ----------------
//**********************************************************************

#define SYNTAX \
"Syntax: adotest <ODBC source> <SQL query> [<username> <password> [<max. records to retrieve>]]\n"


//**********************************************************************
// main
// ----
//**********************************************************************

int main(int argc, char* argv[])
{ int max_records;
  HRESULT hr;
  char dsn[64], query[256], username[128], password[128], s[256];

// Check the arguments

  if (argc == 2)
  { if (lstrcmp(argv[1], "-?") == 0 || lstrcmp(argv[1], "/?") == 0)
    { printf(SYNTAX);
      return 0;
    }
  }

  if (argc != 3 && argc != 5 && argc != 6)
  { printf(SYNTAX);
    return 0;
  }

  lstrcpy(dsn, argv[1]);
  lstrcpy(query, argv[2]);

  lstrcpy(username, "");
  lstrcpy(password, "");
  if (argc == 5)
  { lstrcpy(username, argv[3]);
    lstrcpy(password, argv[4]);
  }

  max_records = 999999;
  if (argc == 6)
  { if (!strtoint(argv[5], &max_records))
    { printf("The third argument \"%s\" is not a valid integer\n", argv[5]);
      return 2;
    }
  }

// Initialise COM

  hr = CoInitialize(NULL);

  if (FAILED(hr))
  { printf("There was an error initialising the COM subsystem:\n%s\n", GetLastErrorMessage());
    return 2;
  }

// Open the ODBC source

  CRhsADOConnection connection;

  try
  { if (argc > 3)
      connection.Open(dsn, username, password);
    else
      connection.Open(dsn);
  }
  catch (CRhsADOException e)
  { printf("Cannot open the ODBC source \"%s\":\n%s\n", dsn, e.GetError(connection));
    connection.Close();
    CoUninitialize();
    return 2;
  }

// Execute the command

  CRhsADORecordset recordset;
  
  try
  { recordset = connection.Execute(query, adCmdText);
  }
  catch (CRhsADOException e)
  { printf("There was an error executing the query \"%s\":\n%s\n", query, e.GetError(connection));
    connection.Close();
    CoUninitialize();
    return 2;
  }

// Print the field names.  We need to get the first record so we have some
// fields to get the name from.

  recordset.MoveFirst();
  if (recordset.GetEOF())
  { printf("Query returned no records\n");
    recordset.Close();
    connection.Close();
    CoUninitialize();
    return 0;
  }

// Loop over the fields printing their names

  int num_fields;
  CRhsADOField field;

  num_fields = recordset.FieldCount();

  for (int i = 0; i < num_fields; i++)
  { if (i > 0) printf("\t");

    field = recordset.GetField(i);

    try
    { field.GetName(s, 255);
    }
    catch (CRhsADOException e)
    { lstrcpy(s, "<No name>");
    }

    printf(s);
  }

  printf("\n");

// Work through the result set

  long num_records = 0;
  CRhsVariant fieldval;
  CRhsDate d;

  recordset.MoveFirst();

  while (num_records < max_records) // Don't retrieve more records than requested
  { if (recordset.GetEOF()) 
      break;
    num_records++;

// Get each field in turn

    for (int i = 0; i < num_fields; i++)
    { field = recordset.GetField(i);
      field.GetValue(fieldval);

// This needs to be a separate function to handle all the many different formats.
// For now just leave it simple.

      if (i > 0)
        printf("\t");

      switch (field.GetType())
      { case adInteger:
        case adSmallInt:
          printf("%i", (int) fieldval);
          break;

        case adDouble:
          printf("%f", (double) fieldval);
          break;

        case adChar:
        case adVarChar:
          printf("%s", (char*) fieldval.AsChar(s, 255));
          break;

        case adBoolean:
          printf("%s", (int) fieldval ? "TRUE" : "FALSE");
          break;

        case adDBTimeStamp:
          d.SetDate((double) fieldval);
          printf("%s", d.Format(s, 255, 4));
          break;

        default:
          printf("Unrecognised field type %i", field.GetType());
      }
    }

    field.Close();

// Next record

    printf("\n");

    recordset.MoveNext();
  }

// Result set finished

  printf("Number of records = %i\n", num_records);

  recordset.Close();

// Free the connection

  connection.Close();

// All done

  CoUninitialize();

  return 0;
}


//**********************************************************************
// GetLastErrorMessage
// -------------------
//**********************************************************************

const char* GetLastErrorMessage(void)
{ DWORD d;
  static char errmsg[1024];

  d = GetLastError();

  strcpy(errmsg, "<unknown error>");
  FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, d, 0, errmsg, 1023, NULL);
  return(errmsg);
}


//**********************************************************************
// strtoint
// --------
// Convert string to int
//**********************************************************************

BOOL strtoint(const char* s, int* i)
{ const char* start;
  char* end;

  for (start = s; *start == ' ' || *start == '\t'; start++);
  *i = (int) strtol(start, &end, 10);

  return(end > start ? TRUE : FALSE);
}


